@@ -293,7 +293,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             } while (c != EOF);
             *p='\0';
             p=options;
-            while (isspace((int) ((unsigned char) c)) != 0)
+            while (isspace(c) != 0)
               c=ReadBlobByte(image);
             if (c == (int) '=')
               {
@@ -325,7 +325,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                         }
                     }
                   if (*options != '{')
-                    if (isspace((int) ((unsigned char) c)) != 0)
+                    if (isspace(c) != 0)
                       break;
                 }
                 if (options == (char *) NULL)
@@ -758,7 +758,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
         else
           c=ReadBlobByte(image);
-      while (isspace((int) ((unsigned char) c)) != 0)
+      while (isspace(c) != 0)
         c=ReadBlobByte(image);
     }
     options=DestroyString(options);
@@ -1035,7 +1035,7 @@ ModuleExport size_t RegisterMPCImage(void)
 
   entry=SetMagickInfo("CACHE");
   entry->description=ConstantString("Magick Persistent Cache image format");
-  entry->module=ConstantString("MPC");
+  entry->magick_module=ConstantString("MPC");
   entry->stealth=MagickTrue;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("MPC");
@@ -1044,7 +1044,7 @@ ModuleExport size_t RegisterMPCImage(void)
   entry->magick=(IsImageFormatHandler *) IsMPC;
   entry->description=ConstantString("Magick Persistent Cache image format");
   entry->seekable_stream=MagickTrue;
-  entry->module=ConstantString("MPC");
+  entry->magick_module=ConstantString("MPC");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
