@@ -670,7 +670,15 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
     }
   if (info.profile != (StringInfo *) NULL)
-    (void) SetImageProfile(image,"xmp",info.profile);
+    {
+      char
+        *profile;
+
+      (void) SetImageProfile(image,"xmp",info.profile);
+      profile=(char *) GetStringInfoDatum(info.profile);
+      if (strstr(profile,"Adobe Illustrator") != (char *) NULL)
+        (void) CopyMagickString(image->magick,"AI",MagickPathExtent);
+    }
   CleanupPDFInfo(&info);
   (void) CloseBlob(image);
   if (image_info->number_scenes != 0)
@@ -748,7 +756,7 @@ ModuleExport size_t RegisterPDFImage(void)
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Adobe Illustrator CS2");
   entry->mime_type=ConstantString("application/pdf");
-  entry->module=ConstantString("PDF");
+  entry->magick_module=ConstantString("PDF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("EPDF");
   entry->decoder=(DecodeImageHandler *) ReadPDFImage;
@@ -758,7 +766,7 @@ ModuleExport size_t RegisterPDFImage(void)
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Encapsulated Portable Document Format");
   entry->mime_type=ConstantString("application/pdf");
-  entry->module=ConstantString("PDF");
+  entry->magick_module=ConstantString("PDF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PDF");
   entry->decoder=(DecodeImageHandler *) ReadPDFImage;
@@ -768,7 +776,7 @@ ModuleExport size_t RegisterPDFImage(void)
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Portable Document Format");
   entry->mime_type=ConstantString("application/pdf");
-  entry->module=ConstantString("PDF");
+  entry->magick_module=ConstantString("PDF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PDFA");
   entry->decoder=(DecodeImageHandler *) ReadPDFImage;
@@ -778,7 +786,7 @@ ModuleExport size_t RegisterPDFImage(void)
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Portable Document Archive Format");
   entry->mime_type=ConstantString("application/pdf");
-  entry->module=ConstantString("PDF");
+  entry->magick_module=ConstantString("PDF");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
