@@ -134,7 +134,7 @@ ModuleExport size_t RegisterPS3Image(void)
   entry->encoder=(EncodeImageHandler *) WritePS3Image;
   entry->description=ConstantString("Level III Encapsulated PostScript");
   entry->mime_type=ConstantString("application/postscript");
-  entry->module=ConstantString("PS3");
+  entry->magick_module=ConstantString("PS3");
   entry->seekable_stream=MagickTrue;
   entry->blob_support=MagickFalse;
   (void) RegisterMagickInfo(entry);
@@ -142,7 +142,7 @@ ModuleExport size_t RegisterPS3Image(void)
   entry->encoder=(EncodeImageHandler *) WritePS3Image;
   entry->description=ConstantString("Level III PostScript");
   entry->mime_type=ConstantString("application/postscript");
-  entry->module=ConstantString("PS3");
+  entry->magick_module=ConstantString("PS3");
   entry->seekable_stream=MagickTrue;
   entry->blob_support=MagickFalse;
   (void) RegisterMagickInfo(entry);
