@@ -139,6 +139,9 @@ MagickExport Image *ConstituteImage(const size_t columns,
   register ssize_t
     i;
 
+  size_t
+    length;
+
   /*
     Allocate image structure.
   */
@@ -150,7 +153,8 @@ MagickExport Image *ConstituteImage(const size_t columns,
   image=AcquireImage((ImageInfo *) NULL);
   if (image == (Image *) NULL)
     return((Image *) NULL);
-  for (i=0; i < (ssize_t) strlen(map); i++)
+  length=strlen(map);
+  for (i=0; i < (ssize_t) length; i++)
   {
     switch (map[i])
     {
@@ -182,7 +186,7 @@ MagickExport Image *ConstituteImage(const size_t columns,
       }
       default:
       {
-        if (strlen(map) == 1)
+        if (length == 1)
           image->colorspace=GRAYColorspace;
         break;
       }
@@ -632,29 +636,17 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       "notify the developers",image->magick,exception->severity);
   if (IsBlobTemporary(image) != MagickFalse)
     (void) RelinquishUniqueFileResource(read_info->filename);
-  if (IsSceneGeometry(read_info->scenes,MagickFalse) != MagickFalse)
+  if ((IsSceneGeometry(read_info->scenes,MagickFalse) != MagickFalse) &&
+      (GetImageListLength(image) != 1))
     {
-      int 
-        first_scene,
-        last_scene,
-        n;
-
       Image
         *clones;
 
-      n=sscanf(read_info->scenes,"%d-%d",&first_scene,&last_scene);
-      if (n != 2)
+      clones=CloneImages(image,read_info->scenes,exception);
+      if (clones != (Image *) NULL)
         {
-          clones=CloneImages(image,read_info->scenes,exception);
-          if (clones == (Image *) NULL)
-            (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
-              "SubimageSpecificationReturnsNoImages","`%s'",
-              read_info->filename);
-          else
-            {
-              image=DestroyImageList(image);
-              image=GetFirstImageInList(clones);
-            }
+          image=DestroyImageList(image);
+          image=GetFirstImageInList(clones);
         }
     }
   for (next=image; next != (Image *) NULL; next=GetNextImageInList(next))
